///**********************************************************************************************************************************
///  GCGradient.h
///  GCDrawKit
///
///  Created by graham on 2/03/05.
///  Released under the Creative Commons license 2006 Apptree.net.
///
/// 
///  This work is licensed under the Creative Commons Attribution-ShareAlike 2.5 License.
///  To view a copy of this license, visit http://creativecommons.org/licenses/by-sa/2.5/ or send a letter to
///  Creative Commons, 543 Howard Street, 5th Floor, San Francisco, California, 94105, USA.
///
///**********************************************************************************************************************************

#import <Cocoa/Cocoa.h>

// gradient type:

typedef enum
{
	kGCGradientTypeLinear		= 0,
	kGCGradientTypeRadial		= 1,
	kGCGradientSweptAngle		= 2
}
DKGradientType;

// gradient blending mode:

typedef enum
{
	kGCGradientRGBBlending			= 0,
	kGCGradientHSBBlending			= 1
}
DKGradientBlending;


typedef enum
{
	kGCGradientInterpLinear			= 0,
	kGCGradientInterpQuadratic		= 2,
	kGCGradientInterpCubic			= 3,
	kGCGradientInterpSinus			= 4
}
DKGradientInterpolation;



@class GCColorStop;

@interface GCGradient : NSObject <NSCopying, NSCoding>

// useful UI presets

+ (GCGradient*)			aquaSelectedGradient;
+ (GCGradient*)			aquaNormalGradient;
+ (GCGradient*)			aquaPressedGradient;

+ (GCGradient*)			unifiedSelectedGradient;
+ (GCGradient*)			unifiedNormalGradient;
+ (GCGradient*)			unifiedPressedGradient;
+ (GCGradient*)			unifiedDarkGradient;


+ (GCGradient*)			sourceListSelectedGradient;
+ (GCGradient*)			sourceListUnselectedGradient;

// simple gradient convenience methods

+ (GCGradient*)			defaultGradient;
+ (GCGradient*)			gradientWithStartingColor:(NSColor*) c1 endingColor:(NSColor*) c2;
+ (GCGradient*)			gradientWithStartingColor:(NSColor*) c1 endingColor:(NSColor*) c2 type:(int) gt angle:(float) degrees;

// here's some we made earlier: named gradients

+ (GCGradient*)			gradientWithName:(NSString*) name;
+ (void)				registerGradient:(GCGradient*) grad withName:(NSString*) name;
+ (NSArray*)			registeredGradients;

// modified copies:

- (GCGradient*)			gradientByColorizingWithColor:(NSColor*) color;
- (GCGradient*)			gradientWithAlpha:(float) alpha;

// find the color at a given value 0..1

- (NSColor*)			colorAtValue:(float) val;
 
// a variety of ways to fill a path

- (void)				fillRect:(NSRect)rect;
- (void)				fillPath:(NSBezierPath*) path;
- (void)				fillPath:(NSBezierPath*) path centreOffset:(NSPoint) co;
- (void)				fillPath:(NSBezierPath*) path startingAtPoint:(NSPoint) sp
									startRadius:(float) sr endingAtPoint:(NSPoint) ep endRadius:(float) er;

// setting stops

- (GCColorStop*)		addColor:(NSColor*) Color at:(float) pos;
- (void)				removeColorStop:(GCColorStop*) stop;
- (void)				removeLastColor;
- (void)				removeAllColors;

// setting the angle

- (void)				setAngle:(float) ang;
- (float)				angle;

- (void)				setAngleInDegrees:(float) degrees;
- (float)				angleInDegrees;

// setting gradient type, blending and interpolation settings

- (void)				setGradientType:(DKGradientType) gt;
- (DKGradientType)		gradientType;

- (void)				setGradientBlending:(DKGradientBlending) bt;
- (DKGradientBlending)  gradientBlending;

- (void)				setGradientInterpolation:(DKGradientInterpolation) intrp;
- (DKGradientInterpolation)	gradientInterpolation;

// swatch images

- (NSImage*)			swatchImageWithSize:(NSSize) size withBorder:(BOOL) showBorder;
- (NSImage*)			standardSwatchImage;

// pasteboard

+ (BOOL)				canInitalizeFromPasteboard:(NSPasteboard*) pboard;
+ (GCGradient*)			gradientWithPasteboard:(NSPasteboard*) pboard;
+ (void)				setPasteboardImageSize:(NSSize) pbiSize;

- (BOOL)				writeToPasteboard:(NSPasteboard*) pboard;
- (BOOL)				writeFileToPasteboard:(NSPasteboard*) pboard;

// converting to/from other formats

- (NSDictionary*)		plistRepresentation;
- (NSData*)				fileRepresentation;
- (NSData*)				pdf;

+ (GCGradient*)			gradientWithPlist:(NSDictionary*) plist;

// file interface

+ (GCGradient*)			gradientWithContentsOfFile:(NSString*) path;
- (BOOL)				writeToFile:(NSString*) path atomically:(BOOL) flag;

@end

// constants

extern NSString*	GPGradientPasteboardType;

// notifications sent by GCGradient:

extern NSString*	kGCNotificationGradientWillAddColorStop;
extern NSString*	kGCNotificationGradientDidAddColorStop;
extern NSString*	kGCNotificationGradientWillRemoveColorStop;
extern NSString*	kGCNotificationGradientDidRemoveColorStop;
extern NSString*	kGCNotificationGradientWillChange;
extern NSString*	kGCNotificationGradientDidChange;
